@@ -9,8 +9,9 @@
 """
 
 __author__ = "Ryan McGrath <ryan@venodesigns.net>"
-__version__ = "1.4.1"
+__version__ = "1.4.2"
 
+import cgi
 import urllib
 import urllib2
 import urlparse
@@ -100,7 +101,7 @@ class AuthError(TwythonError):
 
 
 class Twython(object):
-	def __init__(self, twitter_token = None, twitter_secret = None, oauth_token = None, oauth_token_secret = None, headers=None, callback_url=None):
+	def __init__(self, twitter_token = None, twitter_secret = None, oauth_token = None, oauth_token_secret = None, headers=None, callback_url=None, client_args={}):
 		"""setup(self, oauth_token = None, headers = None)
 
 			Instantiates an instance of Twython. Takes optional parameters for authentication and such (see below).
@@ -112,6 +113,7 @@ class Twython(object):
 					pass it in and it'll be used for all requests going forward.
 				oauth_token_secret - see oauth_token; it's the other half.
 				headers - User agent header, dictionary style ala {'User-Agent': 'Bert'}
+				client_args - additional arguments for HTTP client (see httplib2.Http.__init__), e.g. {'timeout': 10.0}
 
 				** Note: versioning is not currently used by search.twitter functions; when Twitter moves their junk, it'll be supported.
 		"""
@@ -142,12 +144,12 @@ class Twython(object):
 
 		# Filter down through the possibilities here - if they have a token, if they're first stage, etc.
 		if consumer is not None and token is not None:
-			self.client = oauth.Client(consumer, token)
+			self.client = oauth.Client(consumer, token, **client_args)
 		elif consumer is not None:
-			self.client = oauth.Client(consumer)
+			self.client = oauth.Client(consumer, **client_args)
 		else:
 			# If they don't do authentication, but still want to request unprotected resources, we need an opener.
-			self.client = httplib2.Http()
+			self.client = httplib2.Http(**client_args)
 
 	def __getattr__(self, api_call):
 		"""
@@ -176,7 +178,7 @@ class Twython(object):
 
 			# Then open and load that shiiit, yo. TODO: check HTTP method and junk, handle errors/authentication
 			if fn['method'] == 'POST':
-				resp, content = self.client.request(base, fn['method'], urllib.urlencode(dict([k, v.encode('utf-8')] for k, v in kwargs.items())), headers = self.headers)
+				resp, content = self.client.request(base, fn['method'], urllib.urlencode(dict([k, Twython.encode(v)] for k, v in kwargs.items())), headers = self.headers)
 			else:
 				url = base + "?" + "&".join(["%s=%s" %(key, value) for (key, value) in kwargs.iteritems()])
 				resp, content = self.client.request(url, fn['method'], headers = self.headers)
@@ -205,7 +207,10 @@ class Twython(object):
 		if resp['status'] != '200':
 			raise AuthError("Seems something couldn't be verified with your OAuth junk. Error: %s, Message: %s" % (resp['status'], content))
 		
-		request_tokens = dict(urlparse.parse_qsl(content))
+		try:
+			request_tokens = dict(urlparse.parse_qsl(content))
+		except:
+			request_tokens = dict(cgi.parse_qsl(content))
 		
 		oauth_callback_confirmed = request_tokens.get('oauth_callback_confirmed')=='true'
 		
@@ -233,8 +238,11 @@ class Twython(object):
 			Returns authorized tokens after they go through the auth_url phase.
 		"""
 		resp, content = self.client.request(self.access_token_url, "GET")
-		return dict(urlparse.parse_qsl(content))
-
+		try:
+			return dict(urlparse.parse_qsl(content))
+		except:
+			return dict(cgi.parse_qsl(content))
+	
 	# ------------------------------------------------------------------------------------------------------------------------
 	# The following methods are all different in some manner or require special attention with regards to the Twitter API.
 	# Because of this, we keep them separate from all the other endpoint definitions - ideally this should be change-able,
@@ -463,3 +471,9 @@ class Twython(object):
 		except:
 			pass
 		return text
+
+	@staticmethod
+	def encode(text):
+		if isinstance(text, (str,unicode)):
+			return Twython.unicode2utf8(text)
+		return str(text)
